package junior;
import junior.core.*;

public class JrBase 
{
   final static public Object NO_VALUE = DummyValues.NONE;
   final static public Object NULL_VALUE = DummyValues.NULL;
   // Constant wrappers
   static public IdentifierWrapper ConstWrapper(Identifier id){ return new ConstIdentifierWrapper(id); }
   static public ObjectWrapper ConstWrapper(Object obj){ return new ConstObjectWrapper(obj); }
   static public IntegerWrapper ConstWrapper(int n){ return new ConstIntegerWrapper(n); }
   static public BooleanWrapper ConstWrapper(boolean b){ return new ConstBooleanWrapper(b); }
   // Machines
   static public UnSyncMachine UnSyncMachine(Program p) { return new BasicContext(p); }
   static public SyncMachine SyncMachine(Program p) { return new ExecContext(p); } 
   // Atom	
   static public Program Atom(Action action){ return new Atom(action); }
   // Nothing, Stop, Seq, Par, Loop   
   static public Program Nothing(){ return new Nothing(); }
   static public Program Stop(){ return new Stop(); }  
   static public Program Seq(Program first, Program second){ return new Seq(first,second); }
   static public Program Par(Program left, Program right){ return new Merge(left,right); }
   static public Program Loop(Program body){ return new Loop(body); }
   // Repeat
   static public Program Repeat(IntegerWrapper wrapper, Program body){ return new Repeat(wrapper,body); }
   // If
   static public Program If(BooleanWrapper cond, Program thenInst, Program elseInst){ return new If(cond,thenInst,elseInst); }
   // Generate
   static public Program Generate(IdentifierWrapper iwrap,ObjectWrapper owrap){ return new Generate(iwrap,owrap); }    
   // Await, Until, When      
   static public Program Await(Configuration config){ return new Await(config); }
   static public Program Until(Configuration config, Program body, Program handler){ return new Until(config,body,handler); }
   static public Program When(Configuration config, Program thenInst, Program elseInst){ return new When(config,thenInst,elseInst); }
   // Local   
   static public Program Local(Identifier event, Program body){ return new Local(event,new EventDataImpl(),body); }
   // Control 
   static public Program Control(IdentifierWrapper wrapper, Program body){ return new Control(new Presence(wrapper),body); }
   // Freezable      
   static public Program Freezable(IdentifierWrapper wrapper, Program body){ return new Freezable(new Presence(wrapper),body); }
   // Link   
   static public Program Link(ObjectWrapper wrapper, Program body){ return new Link(wrapper,body); } 
   // Configurations  
   static public Configuration Presence(IdentifierWrapper wrapper){ return new Presence(wrapper); }
   static public Configuration And(Configuration c1, Configuration c2){ return new And(c1,c2); }
   static public Configuration Or(Configuration c1, Configuration c2){ return new Or(c1,c2); }
   static public Configuration Not(Configuration c){ return new Not(c); }
}
