package junior.core;
import junior.*;
 
public class Merge extends BinaryInstruction
{
  final public byte leftFlag, rightFlag;
  // constructors
  public Merge(Program left, Program right, byte leftFlag, byte rightFlag){ 
     super(left,right); this.leftFlag = leftFlag; this.rightFlag = rightFlag;
  }
  public Merge(Program left,Program right){ this(left,right,SUSP,SUSP); } 
  
  public String toString(){ return PrintInstruction.Merge(""+left,""+right); }
  // determine the result
  public Instruction newTerm(Instruction l, Instruction r, byte lf, byte rf){
     return new Merge(l,r,lf,rf);
  }
  
  public MicroState result(Instruction l, Instruction r, byte lf, byte rf){
      byte b = SUSP, nlf = lf, nrf = rf;
     if(lf != SUSP && rf != SUSP){ 
        b = (lf==TERM && rf==TERM) ? TERM : STOP;
        if (lf==STOP) nlf = SUSP;
        if (rf==STOP) nrf = SUSP;
     }
     return new MicroState(b,newTerm(l,r,nlf,nrf));
  }
  // rewriting
  public MicroState rewrite(EnvironmentImpl env){
     if (leftFlag == SUSP && rightFlag != SUSP){ 
        MicroState s = left.rewrite(env);
        return result(s.term,right,s.flag,rightFlag);
     }
     if (leftFlag != SUSP && rightFlag == SUSP){ 
        MicroState s = right.rewrite(env);
        return result(left,s.term,leftFlag,s.flag);
     }
     // Both branches are suspended     
     MicroState ls = left.rewrite(env), rs = right.rewrite(env);
     return result(ls.term,rs.term,ls.flag,rs.flag);
  }
  // copy; don't forget the flags !
  public Program copy(){ return new Merge(left.copy(),right.copy(),leftFlag,rightFlag); }
}
